function [c, ceq, dc, dceq] = GMMMPEC_c(x0) 

% GMMMPEC_c
% Constraints for the random coefficients Logit estimated via MPEC.
%
% source: Dube, Fox and Su (2012)
% Code Revised: January 2012
% Code Modified by P. Ketz: December 2015  


global x x_rc IV K K_rc prods T share 
global prodsMarket numProdsTotal marketStarts marketEnds
global points weights num_points

mu = x0(1:K,1);                                 % mean tastes
sigma = sqrt(x0(K+1:K+K_rc,1));                  % st. deviation of tastes
delta = x0(K+K_rc+1:K+K_rc+numProdsTotal,1);    % mean utilities
g = x0(K+K_rc+numProdsTotal+1:end,1);           % moment condition values

cong = g - IV'*(delta - x*mu);                  % constraints on moment conditions

expmu = exp(x_rc*diag(sigma)*points);           % exponentiated deviations from mean utilities
expmeanval = exp(delta);
[EstShare, simShare] = ind_shnorm(expmeanval,expmu);

c = [];
dc = [];

ceq = [EstShare - share ;
     cong ]; 
 
if nargout>2
    ng = size(g,1);
    ooo = ones(1,K_rc);
    
    % Evaluate the Gradients
    dSdsigma = zeros(numProdsTotal,K_rc);
    dSddeltaDIAG = zeros(numProdsTotal,prods);
    dSddelta = zeros(numProdsTotal, numProdsTotal);
    
    for t=1:T,
        index = marketStarts(t):marketEnds(t);
        ooo1 = ones(prodsMarket(t),1);
        for rr = 1:num_points,
            dSddeltaDIAG(index,1:prodsMarket(t)) = dSddeltaDIAG(index,1:prodsMarket(t)) + (diag(simShare(index,rr)) - simShare(index,rr)*simShare(index,rr)')*weights(1,rr);
            dSdsigma(index,:) = dSdsigma(index,:) + (simShare(index,rr)*ooo).*(ooo1*points(:,rr)').*( x_rc(index,:) - (ooo1*(simShare(index,rr)'*x_rc(index,:))))*weights(1,rr);
        end
        dSddelta(index,index) = dSddeltaDIAG(index,1:prodsMarket(t));
    end

    dceq1 = [zeros(numProdsTotal,K), dSdsigma./(ones(numProdsTotal,1)*sigma')/2, dSddelta, zeros(numProdsTotal,ng)];
    dceq2 = [IV'*x, zeros(ng, K_rc), -IV', eye(ng)];
    dceq = [dceq1; dceq2]';
end