function [hess] = GMMMPEC_hess(x0,lambda) 

% GMMMPEC_c
% Constraints for the random coefficients Logit estimated via MPEC.
%
% source: Dube, Fox and Su (2012)
% Code Revised: January 2012
% Code Modified by P. Ketz: December 2015  


global x_rc K K_rc prods T W 
global prodsMarket numProdsTotal marketStarts marketEnds
global points weights num_points

mu = x0(1:K,1);                                 % mean tastes
sigma = x0(K+1:K+K_rc,1).^0.5;                  % st. deviation of tastes
delta = x0(K+K_rc+1:K+K_rc+numProdsTotal,1);    % mean utilities
g = x0(K+K_rc+numProdsTotal+1:end,1);           % moment condition values

nx0 = size(x0,1);
ooo = ones(1,K_rc);

hess = zeros(nx0,nx0);
hess(K+K_rc+numProdsTotal+1:end,K+K_rc+numProdsTotal+1:end) = 2*W;

expmu = exp(x_rc*diag(sigma)*points);    
expmeanval = exp(delta);
[EstShare, simShare] = ind_shnorm(expmeanval,expmu);

dL2dsigma2 = zeros(K_rc,K_rc);
dL2ddeltadsigma = zeros(numProdsTotal, K_rc);
dL2ddelta2DIAG = zeros(numProdsTotal,prods);
dL2ddelta2 = zeros(numProdsTotal, numProdsTotal);

% Evaluate the hessian    
for t=1:T,
    index = marketStarts(t):marketEnds(t);
    multip = lambda.eqnonlin(index);
    ooo1 = ones(prodsMarket(t),1);
    for rr = 1:num_points,
            
        simS = simShare(index,rr);
        sumprod_mpsimS = multip'*simS;
            
        blk1 = sumprod_mpsimS*(-diag(simS) + 2*simS*simS');          
        blk2 = -(multip.*simS)*simS';
        blk3 = diag(multip.*simS);
        blk = blk1 + blk2 + blk2'+blk3;        
        dL2ddelta2DIAG(index,1:prodsMarket(t)) = dL2ddelta2DIAG(index,1:prodsMarket(t)) + blk*weights(1,rr);
        
        xsimSx = x_rc(index,:) - (ooo1*(simS'*x_rc(index,:)));
        xsimSxv = xsimSx.*(ooo1*points(:,rr)');
        dSdsigmarr = (simS*ooo).*xsimSxv;
     
        dL2ddeltadsigmarr = -simS*multip'*dSdsigmarr - sumprod_mpsimS*dSdsigmarr + (multip*ooo).*dSdsigmarr;       
        dL2ddeltadsigma(index,:) = dL2ddeltadsigma(index,:) + dL2ddeltadsigmarr*weights(1,rr);
        dL2dsigma2rr = ((multip*ooo).*dSdsigmarr)'*xsimSxv + sumprod_mpsimS*(-dSdsigmarr'*x_rc(index,:).*(ones(K_rc,1)*points(:,rr)'));
        tool = sigma*ones(1,K_rc);
        dL2dsigma2rr = dL2dsigma2rr./tool./tool'/4;
        dL2dsigma2 = dL2dsigma2 + dL2dsigma2rr*weights(1,rr) - diag((multip'*dSdsigmarr)./sigma'.^3/4)*weights(1,rr);
        
    end
    dL2ddelta2(index,index) = dL2ddelta2DIAG(index,1:prodsMarket(t));
end

hess(K+1:K+K_rc,K+1:K+K_rc) = dL2dsigma2;        
hess(K+K_rc+1:K+K_rc+numProdsTotal,K+1:K+K_rc) = dL2ddeltadsigma./(ones(numProdsTotal,1)*sigma')/2;    
hess(K+1:K+K_rc,K+K_rc+1:K+K_rc+numProdsTotal) = (dL2ddeltadsigma./(ones(numProdsTotal,1)*sigma')/2)';        
hess(K+K_rc+1:K+K_rc+numProdsTotal,K+K_rc+1:K+K_rc+numProdsTotal) = dL2ddelta2;
    