function Ddelta = jacobian(sigma2,delta,x_rc)

% JACOBIAN
% Computes the Jacobian of the mean utilities associated with
% the random coefficients Logit demand model.
%
% source: Dube, Fox and Su (2012)
% Code Revised: January 2012
% Code Modified by P. Ketz: December 2015  


global prods T K_rc 
global prodsMarket numProdsTotal marketStarts marketEnds
global points weights num_points

sigma = sqrt(sigma2);

expmu = exp(x_rc*diag(sigma)*points);      % exponentiated deviations from mean utilities
expmeanval = exp(delta);
[EstShare, simShare] = ind_shnorm(expmeanval,expmu);

ooo = ones(1,K_rc);

dSdsigma = zeros(numProdsTotal,K_rc);
dSddeltaDIAG = zeros(numProdsTotal,prods);
dSddelta = zeros(numProdsTotal, numProdsTotal);

for t=1:T,
    index = marketStarts(t):marketEnds(t);
    ooo1 = ones(prodsMarket(t),1);
    for rr = 1:num_points,
        dSddeltaDIAG(index,1:prodsMarket(t)) = dSddeltaDIAG(index,1:prodsMarket(t)) + (diag(simShare(index,rr)) - simShare(index,rr)*simShare(index,rr)')*weights(1,rr);
        dSdsigma(index,:) = dSdsigma(index,:) + (simShare(index,rr)*ooo).*(ooo1*points(:,rr)').*( x_rc(index,:) - (ooo1*(simShare(index,rr)'*x_rc(index,:))))*weights(1,rr);
    end
    dSddelta(index,index) = dSddeltaDIAG(index,1:prodsMarket(t));
end

Ddelta = -inv(dSddelta)*dSdsigma./(ones(numProdsTotal,1)*sigma')/2;



